"""Nessita Syntax to Docker File.

Example:

    precise:
        apt:
            apache2: 3.3-4
            bzr:
        pip:
            foobar:
    metadata:
        test_script: ./test


will be converted to

    from ubuntu:precise
    run apt-get -q -y install apache2=3.3-4
    run apt-get -q -y install bzr
    run pip install --download=/tmp/pipcache --no-install foobar
    run ip link set dev eth0 down
    run pip install --find-links=file:///tmp/pipcache --no-index foobar

and also the system will use "./test" as script
"""

import yaml

# the conversion between our nice base names and those that are
# needed by docker
BASE_TRANSLATIONS = {
    'precise': 'ubuntu:precise',
}

def _get_base(config, **params):
    """Process the system base stuff."""
    # support only one base for now
    if len(config) > 1:
        raise NotImplementedError("Too many base machines.")
    base = config.keys()[0]
    return ["from " + BASE_TRANSLATIONS[base]]


def _get_depends_prev(config, **params):
    """All the dependencies before securing machine."""
    # support only one base for now
    config = config.values()[0]
    dependencies = []

    # apt
    items = config.get('apt', [])
    if items:
        for name, ver in sorted(items.items()):
            if ver is None:
                p = name
            else:
                p = "%s=%s" % (name, ver)
            dep = "run apt-get -q -y install " + p
            dependencies.append(dep)

    # pip
    items = config.get('pip', [])
    pip_cache_dir = params["pip_cache_dir"]
    if items:
        for name, ver in sorted(items.items()):
            if ver is None:
                p = name
            else:
                p = "%s==%s" % (name, ver)
            dep = "run pip install --download=%s --no-install %s" % (
                pip_cache_dir, p)
            dependencies.append(dep)

    return dependencies

def _get_secure(config, **params):
    """All that needs to be done to secure the machine."""
    return ["run ip link set dev eth0 down"]

def _get_depends_post(config, **params):
    """All the dependencies stuff after the machine was secured."""
    # support only one base for now
    config = config.values()[0]
    dependencies = []

    # pip
    items = config.get('pip', [])
    pip_cache_dir = params["pip_cache_dir"]
    if items:
        for name, ver in sorted(items.items()):
            if ver is None:
                p = name
            else:
                p = "%s==%s" % (name, ver)
            dep = "run pip install --find-links=file://%s --no-index %s" % (
                    pip_cache_dir, p)
            dependencies.append(dep)

    return dependencies


def _get_rest(config, **params):
    """The final stuff."""
    return []


def parse(config_text, pip_cache_dir):
    """Convert Nessita Syntax and return the text for Docker."""
    config = yaml.load(config_text)
    metadata = config.pop("metadata")

    # the order for calling these functions is VERY important
    functions = [
        _get_base,
        _get_depends_prev,
        _get_secure,
        _get_depends_post,
        _get_rest,
    ]
    data = []
    for func in functions:
        items = func(config, pip_cache_dir=pip_cache_dir)
        data.extend(items)
    docker = "\n".join(data)

    # get stuff from the metadata
    script = metadata["test_script"]

    return script, docker
