"""Tests for the ns2df convertor."""

import unittest

import ns2df

class DependenciesTestCase(unittest.TestCase):
    """Tests for the dependencies transformer."""

    def test_no_dependencies(self):
        config = dict(somebase={})
        res = ns2df._get_depends_prev(config, pip_cache_dir="/tmp/pipcache")
        self.assertEqual(res, [])
        res = ns2df._get_depends_post(config, pip_cache_dir="/tmp/pipcache")
        self.assertEqual(res, [])

    def test_apt_one(self):
        config = dict(somebase={
            'apt': {'bzr': None}
        })
        res = ns2df._get_depends_prev(config, pip_cache_dir="/tmp/pipcache")
        self.assertEqual(res, ["run apt-get -q -y install bzr"])

    def test_apt_several(self):
        config = dict(somebase={
            'apt': {'bzr': None, 'apache2': None}
        })
        res = ns2df._get_depends_prev(config, pip_cache_dir="/tmp/pipcache")
        self.assertEqual(res, [
            "run apt-get -q -y install apache2",
            "run apt-get -q -y install bzr",
        ])

    def test_apt_versions(self):
        config = dict(somebase={
            'apt': {'bzr': None, 'apache2': '3.3-4'}
        })
        res = ns2df._get_depends_prev(config, pip_cache_dir="/tmp/pipcache")
        self.assertEqual(res, [
            "run apt-get -q -y install apache2=3.3-4",
            "run apt-get -q -y install bzr",
        ])

    def test_pip_prev_one(self):
        config = dict(somebase={
            'pip': {'foo': None}
        })
        res = ns2df._get_depends_prev(config, pip_cache_dir="/tmp/pipcache")
        self.assertEqual(res, [
            "run pip install --download=/tmp/pipcache --no-install foo",
        ])

    def test_pip_prev_several(self):
        config = dict(somebase={
            'pip': {'foo': None, 'bar': None}
        })
        res = ns2df._get_depends_prev(config, pip_cache_dir="/tmp/pipcache")
        self.assertEqual(res, [
            "run pip install --download=/tmp/pipcache --no-install bar",
            "run pip install --download=/tmp/pipcache --no-install foo",
        ])

    def test_pip_prev_versions(self):
        config = dict(somebase={
            'pip': {'foo': None, 'bar': '2.1'}
        })
        res = ns2df._get_depends_prev(config, pip_cache_dir="/tmp/pipcache")
        self.assertEqual(res, [
            "run pip install --download=/tmp/pipcache --no-install bar==2.1",
            "run pip install --download=/tmp/pipcache --no-install foo",
        ])

    def test_pip_post_one(self):
        config = dict(somebase={
            'pip': {'foo': None}
        })
        res = ns2df._get_depends_post(config, pip_cache_dir="/tmp/pipcache")
        self.assertEqual(res, [
            "run pip install --find-links=file:///tmp/pipcache --no-index foo"
        ])

    def test_pip_post_several(self):
        config = dict(somebase={
            'pip': {'foo': None, 'bar': None}
        })
        res = ns2df._get_depends_post(config, pip_cache_dir="/tmp/pipcache")
        self.assertEqual(res, [
            "run pip install --find-links=file:///tmp/pipcache --no-index bar",
            "run pip install --find-links=file:///tmp/pipcache --no-index foo",
        ])

    def test_pip_post_versions(self):
        config = dict(somebase={
            'pip': {'foo': None, 'bar': '2.1'}
        })
        res = ns2df._get_depends_post(config, pip_cache_dir="/tmcache")
        self.assertEqual(res, [
            "run pip install --find-links=file:///tmcache --no-index bar==2.1",
            "run pip install --find-links=file:///tmcache --no-index foo",
        ])

    def test_mixed_prev(self):
        config = dict(somebase={
            'apt': {'bzr': None, 'apache2': '3.3-4'},
            'pip': {'foo': None, 'bar': None},
        })
        res = ns2df._get_depends_prev(config, pip_cache_dir="/tmp/pipcache")
        self.assertEqual(res, [
            "run apt-get -q -y install apache2=3.3-4",
            "run apt-get -q -y install bzr",
            "run pip install --download=/tmp/pipcache --no-install bar",
            "run pip install --download=/tmp/pipcache --no-install foo",
        ])

    def test_mixed_post(self):
        config = dict(somebase={
            'apt': {'bzr': None, 'apache2': '3.3-4'},
            'pip': {'foo': None, 'bar': None},
        })
        res = ns2df._get_depends_post(config, pip_cache_dir="/tmp/pipcache")
        self.assertEqual(res, [
            "run pip install --find-links=file:///tmp/pipcache --no-index bar",
            "run pip install --find-links=file:///tmp/pipcache --no-index foo",
        ])


class BoilerplateTestCase(unittest.TestCase):
    """Tests for the setup and the rest of commands."""

    def test_one_base(self):
        config = dict(precise={})
        res = ns2df._get_base(config)
        self.assertEqual(res, ["from ubuntu:precise"])

    def test_several_bases(self):
        config = dict(base1={}, base2={})
        self.assertRaises(NotImplementedError, ns2df._get_base, config)

    def test_secure_actions(self):
        res = ns2df._get_secure({})
        self.assertEqual(res, ["run ip link set dev eth0 down"])

    def test_rest(self):
        res = ns2df._get_rest({})
        self.assertEqual(res, [])

    def test_all_mixed(self):
        config_text = """
            precise:
                apt:
                    apache2: 3.3-4
                    bzr:
                pip:
                    foobar:
            metadata:
                test_script: foo
        """
        script, docker = ns2df.parse(config_text, "/tmp/pipcache")
        self.assertEqual(script, "foo")
        self.assertEqual(docker,
            "from ubuntu:precise\n"
            "run apt-get -q -y install apache2=3.3-4\n"
            "run apt-get -q -y install bzr\n"
            "run pip install --download=/tmp/pipcache --no-install foobar\n"
            "run ip link set dev eth0 down\n"
            "run pip install --find-links=file:///tmp/pipcache "
                "--no-index foobar"
        )
